@@ -65,6 +65,8 @@ var bufdefaults = map[string]string{
 	"PageDown":       "CursorPageDown",
 	"CtrlPageUp":     "PreviousTab",
 	"CtrlPageDown":   "NextTab",
+	"ShiftPageUp":    "SelectPageUp",
+	"ShiftPageDown":  "SelectPageDown",
 	"Ctrl-g":         "ToggleHelp",
 	"Alt-g":          "ToggleKeyMenu",
 	"Ctrl-r":         "ToggleRuler",
@@ -95,11 +97,13 @@ var bufdefaults = map[string]string{
 	"Esc": "Escape,Deselect,ClearInfo,RemoveAllMultiCursors,UnhighlightSearch",
 
 	// Mouse bindings
-	"MouseWheelUp":   "ScrollUp",
-	"MouseWheelDown": "ScrollDown",
-	"MouseLeft":      "MousePress",
-	"MouseMiddle":    "PastePrimary",
-	"Ctrl-MouseLeft": "MouseMultiCursor",
+	"MouseWheelUp":     "ScrollUp",
+	"MouseWheelDown":   "ScrollDown",
+	"MouseLeft":        "MousePress",
+	"MouseLeftDrag":    "MouseDrag",
+	"MouseLeftRelease": "MouseRelease",
+	"MouseMiddle":      "PastePrimary",
+	"Ctrl-MouseLeft":   "MouseMultiCursor",
 
 	"Alt-n":        "SpawnMultiCursor",
 	"Alt-m":        "SpawnMultiCursorSelect",
@@ -178,8 +182,10 @@ var infodefaults = map[string]string{
 	"Esc": "AbortCommand",
 
 	// Mouse bindings
-	"MouseWheelUp":   "HistoryUp",
-	"MouseWheelDown": "HistoryDown",
-	"MouseLeft":      "MousePress",
-	"MouseMiddle":    "PastePrimary",
+	"MouseWheelUp":     "HistoryUp",
+	"MouseWheelDown":   "HistoryDown",
+	"MouseLeft":        "MousePress",
+	"MouseLeftDrag":    "MouseDrag",
+	"MouseLeftRelease": "MouseRelease",
+	"MouseMiddle":      "PastePrimary",
 }
