@@ -30,6 +30,11 @@ type Cursor struct {
 	// to know what the original selection was
 	OrigSelection [2]Loc
 
+	// The line number where a new trailing whitespace has been added
+	// or -1 if there is no new trailing whitespace at this cursor.
+	// This is used for checking if a trailing whitespace should be highlighted
+	NewTrailingWsY int
+
 	// Which cursor index is this (for multiple cursors)
 	Num int
 }
@@ -38,6 +43,8 @@ func NewCursor(b *Buffer, l Loc) *Cursor {
 	c := &Cursor{
 		buf: b,
 		Loc: l,
+
+		NewTrailingWsY: -1,
 	}
 	c.StoreVisualX()
 	return c
@@ -396,13 +403,26 @@ func (c *Cursor) SelectTo(loc Loc) {
 
 // WordRight moves the cursor one word to the right
 func (c *Cursor) WordRight() {
+	if c.X == util.CharacterCount(c.buf.LineBytes(c.Y)) {
+		c.Right()
+		return
+	}
 	for util.IsWhitespace(c.RuneUnder(c.X)) {
 		if c.X == util.CharacterCount(c.buf.LineBytes(c.Y)) {
-			c.Right()
 			return
 		}
 		c.Right()
 	}
+	if util.IsNonWordChar(c.RuneUnder(c.X)) && !util.IsWhitespace(c.RuneUnder(c.X)) &&
+		util.IsNonWordChar(c.RuneUnder(c.X+1)) {
+		for util.IsNonWordChar(c.RuneUnder(c.X)) && !util.IsWhitespace(c.RuneUnder(c.X)) {
+			if c.X == util.CharacterCount(c.buf.LineBytes(c.Y)) {
+				return
+			}
+			c.Right()
+		}
+		return
+	}
 	c.Right()
 	for util.IsWordChar(c.RuneUnder(c.X)) {
 		if c.X == util.CharacterCount(c.buf.LineBytes(c.Y)) {
@@ -414,6 +434,10 @@ func (c *Cursor) WordRight() {
 
 // WordLeft moves the cursor one word to the left
 func (c *Cursor) WordLeft() {
+	if c.X == 0 {
+		c.Left()
+		return
+	}
 	c.Left()
 	for util.IsWhitespace(c.RuneUnder(c.X)) {
 		if c.X == 0 {
@@ -421,6 +445,17 @@ func (c *Cursor) WordLeft() {
 		}
 		c.Left()
 	}
+	if util.IsNonWordChar(c.RuneUnder(c.X)) && !util.IsWhitespace(c.RuneUnder(c.X)) &&
+		util.IsNonWordChar(c.RuneUnder(c.X-1)) {
+		for util.IsNonWordChar(c.RuneUnder(c.X)) && !util.IsWhitespace(c.RuneUnder(c.X)) {
+			if c.X == 0 {
+				return
+			}
+			c.Left()
+		}
+		c.Right()
+		return
+	}
 	c.Left()
 	for util.IsWordChar(c.RuneUnder(c.X)) {
 		if c.X == 0 {
@@ -431,6 +466,132 @@ func (c *Cursor) WordLeft() {
 	c.Right()
 }
 
+// SubWordRight moves the cursor one sub-word to the right
+func (c *Cursor) SubWordRight() {
+	if c.X == util.CharacterCount(c.buf.LineBytes(c.Y)) {
+		c.Right()
+		return
+	}
+	if util.IsWhitespace(c.RuneUnder(c.X)) {
+		for util.IsWhitespace(c.RuneUnder(c.X)) {
+			if c.X == util.CharacterCount(c.buf.LineBytes(c.Y)) {
+				return
+			}
+			c.Right()
+		}
+		return
+	}
+	if util.IsNonWordChar(c.RuneUnder(c.X)) && !util.IsWhitespace(c.RuneUnder(c.X)) {
+		for util.IsNonWordChar(c.RuneUnder(c.X)) && !util.IsWhitespace(c.RuneUnder(c.X)) {
+			if c.X == util.CharacterCount(c.buf.LineBytes(c.Y)) {
+				return
+			}
+			c.Right()
+		}
+		return
+	}
+	if util.IsSubwordDelimiter(c.RuneUnder(c.X)) {
+		for util.IsSubwordDelimiter(c.RuneUnder(c.X)) {
+			if c.X == util.CharacterCount(c.buf.LineBytes(c.Y)) {
+				return
+			}
+			c.Right()
+		}
+		if util.IsWhitespace(c.RuneUnder(c.X)) {
+			return
+		}
+	}
+	if c.X == util.CharacterCount(c.buf.LineBytes(c.Y)) {
+		return
+	}
+	if util.IsUpperLetter(c.RuneUnder(c.X)) &&
+		util.IsUpperLetter(c.RuneUnder(c.X+1)) {
+		for util.IsUpperAlphanumeric(c.RuneUnder(c.X)) {
+			if c.X == util.CharacterCount(c.buf.LineBytes(c.Y)) {
+				return
+			}
+			c.Right()
+		}
+		if util.IsLowerAlphanumeric(c.RuneUnder(c.X)) {
+			c.Left()
+		}
+	} else {
+		c.Right()
+		for util.IsLowerAlphanumeric(c.RuneUnder(c.X)) {
+			if c.X == util.CharacterCount(c.buf.LineBytes(c.Y)) {
+				return
+			}
+			c.Right()
+		}
+	}
+}
+
+// SubWordLeft moves the cursor one sub-word to the left
+func (c *Cursor) SubWordLeft() {
+	if c.X == 0 {
+		c.Left()
+		return
+	}
+	c.Left()
+	if util.IsWhitespace(c.RuneUnder(c.X)) {
+		for util.IsWhitespace(c.RuneUnder(c.X)) {
+			if c.X == 0 {
+				return
+			}
+			c.Left()
+		}
+		c.Right()
+		return
+	}
+	if util.IsNonWordChar(c.RuneUnder(c.X)) && !util.IsWhitespace(c.RuneUnder(c.X)) {
+		for util.IsNonWordChar(c.RuneUnder(c.X)) && !util.IsWhitespace(c.RuneUnder(c.X)) {
+			if c.X == 0 {
+				return
+			}
+			c.Left()
+		}
+		c.Right()
+		return
+	}
+	if util.IsSubwordDelimiter(c.RuneUnder(c.X)) {
+		for util.IsSubwordDelimiter(c.RuneUnder(c.X)) {
+			if c.X == 0 {
+				return
+			}
+			c.Left()
+		}
+		if util.IsWhitespace(c.RuneUnder(c.X)) {
+			c.Right()
+			return
+		}
+	}
+	if c.X == 0 {
+		return
+	}
+	if util.IsUpperLetter(c.RuneUnder(c.X)) &&
+		util.IsUpperLetter(c.RuneUnder(c.X-1)) {
+		for util.IsUpperAlphanumeric(c.RuneUnder(c.X)) {
+			if c.X == 0 {
+				return
+			}
+			c.Left()
+		}
+		if !util.IsUpperAlphanumeric(c.RuneUnder(c.X)) {
+			c.Right()
+		}
+	} else {
+		for util.IsLowerAlphanumeric(c.RuneUnder(c.X)) {
+			if c.X == 0 {
+				return
+			}
+			c.Left()
+		}
+		if !util.IsAlphanumeric(c.RuneUnder(c.X)) {
+			c.Right()
+		}
+	}
+}
+
 // RuneUnder returns the rune under the given x position
 func (c *Cursor) RuneUnder(x int) rune {
 	line := c.buf.LineBytes(c.Y)
