@@ -17,7 +17,6 @@ import (
 	"regexp"
 	"runtime"
 	"strings"
-	"sync"
 	"time"
 	"unicode/utf8"
 
@@ -27,7 +26,6 @@ import (
 )
 
 var L *lua.LState
-var Lock sync.Mutex
 
 // LoadFile loads a lua file
 func LoadFile(module string, file string, data []byte) error {
@@ -523,21 +521,16 @@ func importErrors() *lua.LTable {
 func importTime() *lua.LTable {
 	pkg := L.NewTable()
 
-	L.SetField(pkg, "After", luar.New(L, time.After))
 	L.SetField(pkg, "Sleep", luar.New(L, time.Sleep))
-	L.SetField(pkg, "Tick", luar.New(L, time.Tick))
 	L.SetField(pkg, "Since", luar.New(L, time.Since))
 	L.SetField(pkg, "FixedZone", luar.New(L, time.FixedZone))
 	L.SetField(pkg, "LoadLocation", luar.New(L, time.LoadLocation))
-	L.SetField(pkg, "NewTicker", luar.New(L, time.NewTicker))
 	L.SetField(pkg, "Date", luar.New(L, time.Date))
 	L.SetField(pkg, "Now", luar.New(L, time.Now))
 	L.SetField(pkg, "Parse", luar.New(L, time.Parse))
 	L.SetField(pkg, "ParseDuration", luar.New(L, time.ParseDuration))
 	L.SetField(pkg, "ParseInLocation", luar.New(L, time.ParseInLocation))
 	L.SetField(pkg, "Unix", luar.New(L, time.Unix))
-	L.SetField(pkg, "AfterFunc", luar.New(L, time.AfterFunc))
-	L.SetField(pkg, "NewTimer", luar.New(L, time.NewTimer))
 	L.SetField(pkg, "Nanosecond", luar.New(L, time.Nanosecond))
 	L.SetField(pkg, "Microsecond", luar.New(L, time.Microsecond))
 	L.SetField(pkg, "Millisecond", luar.New(L, time.Millisecond))
@@ -545,6 +538,15 @@ func importTime() *lua.LTable {
 	L.SetField(pkg, "Minute", luar.New(L, time.Minute))
 	L.SetField(pkg, "Hour", luar.New(L, time.Hour))
 
+	// TODO: these raw Go timer APIs don't provide any synchronization
+	// with micro. Stop exposing them to lua once plugins switch to using
+	// the safer micro.After() interface instead. See issue #3209
+	L.SetField(pkg, "After", luar.New(L, time.After))
+	L.SetField(pkg, "AfterFunc", luar.New(L, time.AfterFunc))
+	L.SetField(pkg, "NewTicker", luar.New(L, time.NewTicker))
+	L.SetField(pkg, "NewTimer", luar.New(L, time.NewTimer))
+	L.SetField(pkg, "Tick", luar.New(L, time.Tick))
+
 	return pkg
 }
 
