@@ -78,7 +78,7 @@ func JobSpawn(cmdName string, cmdArgs []string, onStdout, onStderr, onExit func(
 	go func() {
 		// Run the process in the background and create the onExit callback
 		proc.Run()
-		jobFunc := JobFunction{onExit, string(outbuf.Bytes()), userargs}
+		jobFunc := JobFunction{onExit, outbuf.String(), userargs}
 		Jobs <- jobFunc
 	}()
 
