@@ -137,7 +137,7 @@ func generateFile(filetype, syntax, header string, rules []interface{}) string {
 	output += fmt.Sprintf("detect: \n    filename: \"%s\"\n", strings.Replace(strings.Replace(syntax, "\\", "\\\\", -1), "\"", "\\\"", -1))
 
 	if header != "" {
-		output += fmt.Sprintf("    header: \"%s\"\n", strings.Replace(strings.Replace(header, "\\", "\\\\", -1), "\"", "\\\"", -1))
+		output += fmt.Sprintf("    signature: \"%s\"\n", strings.Replace(strings.Replace(header, "\\", "\\\\", -1), "\"", "\\\"", -1))
 	}
 
 	output += "\nrules:\n"
